unsigned int
shad_compile(int type, const char *src)
{
   int compiled = 0;
   unsigned int shader = glCreateShader(type);

   if (!shader)
     {
        printf("Can't create shader\n");
        abort();
     }
   glShaderSource(shader, 1, &src, NULL);
   glCompileShader(shader);
   glGetShaderiv(shader, GL_COMPILE_STATUS, &compiled);
   if (!compiled)
     {
        int len = 0;

        glGetShaderiv(shader, GL_INFO_LOG_LENGTH, &len);
        if (len)
          {
             char *buf = malloc(len);
             if (buf)
               {
                  glGetShaderInfoLog(shader, len, NULL, buf);
                  printf("Could not compile shader %i:\n"
                         "%s\n", type, buf);
                  abort();
               }
             glDeleteShader(shader);
             shader = 0;
          }
     }
   return shader;
}

static unsigned int
shad_prog_new(const char *src_vert, const char *src_frag)
{
   int linked = 0;
   unsigned int shad_vert = shad_compile(GL_VERTEX_SHADER, src_vert);
   unsigned int shad_frag = shad_compile(GL_FRAGMENT_SHADER, src_frag);
   unsigned int prog = glCreateProgram();
   if (!prog)
     {
        printf("Can't create program\n");
        abort();
     }
   glAttachShader(prog , shad_vert);
   glAttachShader(prog, shad_frag);
   glLinkProgram(prog);
   glGetProgramiv(prog , GL_LINK_STATUS, &linked);
   if (!linked)
     {
        int len = 0;

        glGetProgramiv(prog, GL_INFO_LOG_LENGTH, &len);
        if (len)
          {
             char *buf = malloc(len);
             if (buf)
               {
                  glGetProgramInfoLog(prog, len, NULL, buf);
                  printf("Could not link program:\n"
                         "%s\n", buf);
                  free(buf);
                  abort();
               }
          }
        glDeleteProgram(prog);
        prog = 0;
     }
   return prog;
}
